var connection;
var firstPacket = true;

function map( x,  in_min,  in_max,  out_min,  out_max){
  var rv = (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
  if (out_max > out_min)
  {
      if (rv > out_max) rv = out_max;
      if (rv < out_min) rv = out_min;
  } else {
      if (rv < out_max) rv = out_max;
      if (rv > out_min) rv = out_min;
  }
  return rv;
}

function sendEvent(wsEvent)
{
    var out_event = JSON.stringify(wsEvent);
    console.log("Transmit: " + out_event);
    try
    {
        connection.send(out_event);
    } catch (err) {
        //console.log(err);
    }
}

function profileChange()
{
    var profileSelect = document.getElementById('profileName');
    var profile = profileSelect.value;
    if (profile == "!")
    {
        var newProfileName = prompt("Please enter your profile name", "profile1");
        var newProfile = document.createElement("option");
        newProfile.innerHTML = newProfileName;
        newProfile.value = newProfileName;
        profileSelect.appendChild(newProfile);
        profileSelect.value = newProfileName;
        profile = newProfileName;
    }
    sendEvent({"profile": profile});
    firstPacket = true;
}

function transmitSettings()
{
    if (!firstPacket)
    {
        var channels = Array();
        for (i = 1; i <= 16; i++)
        {
            var obj = {"output": document.getElementById("selectout" + i).value,
                       "inv": document.getElementById("inv" + i).checked,
                       "min": parseInt(document.getElementById("min" + i).value),
                       "max": parseInt(document.getElementById("max" + i).value)};
            channels[i-1] = obj;
        }
        var dataObj = {"channels": channels};
        sendEvent(dataObj);
        console.log(dataObj);
    }
}

function saveSettings()
{
    sendEvent({"save": true, "profile": document.getElementById('profileName').value });
}

function saveDefaultSettings()
{
    sendEvent({"save": true, "profile": "default" });
}

function applySettings()
{
    sendEvent({"apply": true});
}

function connect()
{
    var hostname = location.hostname;
    if (hostname == '')
        hostname = '192.168.34.204';
    var wsUrl = 'ws://' + hostname + ':81/';
    connection = new WebSocket(wsUrl, ['arduino']);

    connection.onopen = function () {
      console.log("Connected to device!");
      firstPacket = true;
      document.getElementById('saveButton').disabled = false;
      document.getElementById('saveDefaultButton').disabled = false;
      document.getElementById('applyButton').disabled = false;
    };
    connection.onerror = function (error) {
      console.log(error);
    };

    //Code for handling incoming Websocket messages from the server
    connection.onmessage = function (e) {
        //console.log("Receive: " + e.data);
        var jsonObject = JSON.parse(e.data);
        if (jsonObject.hasOwnProperty("profiles"))
        {
            var profiles = jsonObject.profiles;
            var profileSelect = document.getElementById('profileName');
            profileSelect.innerHTML = '';
            var newProfile = document.createElement("option");
            newProfile.innerHTML = 'New Profile';
            newProfile.value = '!';
            profileSelect.appendChild(newProfile);
            for(const prof in profiles)
            {
                var profile = profiles[prof];
                var newProfile = document.createElement("option");
                newProfile.innerHTML = profile;
                newProfile.value = profile;
                profileSelect.appendChild(newProfile);
            }
            profileSelect.value = jsonObject.profile;
        }
        if (jsonObject.hasOwnProperty("channels"))
        {
            var profileSelect = document.getElementById('profileName');
            var currentProfile = profileSelect.value;
            if (currentProfile != jsonObject.profile)
            {
                firstPacket = true;
                profileSelect.value = jsonObject.profile;
            }
            var channels = jsonObject.channels;
            //console.log("Recv");
            for (const chan in channels)
            {
                var channel = channels[chan];

                var valueElementId = "val" + (parseInt(chan)+1);
                var valueElement = document.getElementById(valueElementId);
                
                if (firstPacket)
                {
                    console.log("FIRST PACKET");
                    var invElementId = "inv" + (parseInt(chan)+1);
                    var invElement = document.getElementById(invElementId);
                    invElement.checked = channel.inv;
                    
                    var minElementId = "min" + (parseInt(chan)+1);
                    var minElement = document.getElementById(minElementId);
                    minElement.value = channel.min;
                    
                    var maxElementId = "max" + (parseInt(chan)+1);
                    var maxElement = document.getElementById(maxElementId);
                    maxElement.value = channel.max;
                    
                    var outElementId = "out" + (parseInt(chan)+1);
                    var outElement = document.getElementById(outElementId);
                    makeOutSelect(outElement, channel.output);
                }
                valueElement.innerHTML = channel.val;
                
                var progressElementId = "prog" + (parseInt(chan)+1);
                var progressElement = document.getElementById(progressElementId);
                if (progressElement)
                {
                    progressElement.max = 65534;
                    if (channel.axis > 0)
                    {
                        progressElement.classList.remove('red');
                        progressElement.classList.add('green');
                    } else {
                        progressElement.classList.remove('green');
                        progressElement.classList.add('red');
                    }
                    progressElement.value = (channel.axis + 32767);
                }
            }
            if (firstPacket) firstPacket = false;
        }
    };
    connection.onclose = function () {
      console.log('WebSocket connection closed');
      document.getElementById('saveButton').disabled = true;
      document.getElementById('saveDefaultButton').disabled = true;
      document.getElementById('applyButton').disabled = true;
      firstPacket = true;
      setTimeout(() => { connect() }, 3000);
    };
}

function makeOption(select, name, output)
{
    var nameLower = name.toLowerCase();
    var newOption = document.createElement("option");
    newOption.innerHTML = name;
    newOption.value = nameLower;
    if (output == nameLower)
        newOption.selected = true;
    select.appendChild(newOption);
}

function makeOutSelect(outElement, output)
{
  const newSelect = document.createElement("select");
  outElement.innerHTML = '';
  newSelect.id = "select" + outElement.id;
  newSelect.onchange = function() { transmitSettings(); };
  makeOption(newSelect, "", output);
  makeOption(newSelect, "X", output);
  makeOption(newSelect, "Y", output);
  makeOption(newSelect, "Z", output);
  makeOption(newSelect, "RX", output);
  makeOption(newSelect, "RY", output);
  makeOption(newSelect, "RZ", output);
  makeOption(newSelect, "Throttle", output);
  makeOption(newSelect, "Rudder", output);
  makeOption(newSelect, "Slider1", output);
  makeOption(newSelect, "Slider2", output);
  makeOption(newSelect, "Steering", output);
  makeOption(newSelect, "Brake", output);
  makeOption(newSelect, "Accelerator", output);
  makeOption(newSelect, "Button1", output);
  makeOption(newSelect, "Button2", output);
  makeOption(newSelect, "Button3", output);
  makeOption(newSelect, "Button4", output);
  makeOption(newSelect, "Button5", output);
  makeOption(newSelect, "Button6", output);
  makeOption(newSelect, "Button7", output);
  makeOption(newSelect, "Button8", output);
  makeOption(newSelect, "Start", output);
  makeOption(newSelect, "Select", output);
  outElement.appendChild(newSelect);
}

window.onload = function ()
{
    /*
        <tr style="background-color: #EEEEEE;">
            <td>1 X/Roll</td>
            <td id="val1"></td>
            <td><progress class="red" id="prog1" style="height: 32px; width: 100%;"></progress></td>
            <td><input type="checkbox" id="inv1" onchange="transmitSettings()" /></td>
            <td><input type="number" style="width: 64px;" value="0" id="min1" onchange="transmitSettings()" /></td>
            <td><input type="number" style="width: 64px;" value="0" id="max1" onchange="transmitSettings()" /></td>
            <td id="out1"></td>
        </tr>
    */
    for(var i = 0; i < 16; i++)
    {
        var chanNum = i+1;
        var newChannel = document.createElement("tr");
        
        var rowTitle = document.createElement("td");
        if (chanNum == 1)
            rowTitle.innerHTML = chanNum + ' X/Roll';
        else if (chanNum == 2)
            rowTitle.innerHTML = chanNum + ' Y/Pitch';
        else if (chanNum == 3)
            rowTitle.innerHTML = chanNum + ' Throttle';
        else if (chanNum == 4)
            rowTitle.innerHTML = chanNum + ' Yaw/Rudder';
        else
            rowTitle.innerHTML = chanNum;
        newChannel.appendChild(rowTitle);
        
        var rowProgress = document.createElement("td");
        var rowProg = document.createElement("progress");
        rowProg.classList.add("red");
        rowProg.id = "prog"+chanNum;
        rowProg.style.height = '32px';
        rowProg.style.width = '100%';
        rowProgress.appendChild(rowProg);
        newChannel.appendChild(rowProgress);
        
        var rowMin = document.createElement("td");
        var minNumber = document.createElement("input");
        minNumber.id = "min"+chanNum;
        minNumber.type = "number";
        minNumber.style.width = '64px';
        minNumber.onchange = function() { transmitSettings(); };
        rowMin.appendChild(minNumber);
        var minButton = document.createElement("button");
        minButton.onclick = function()
        {
            var chanNum = this.id.substring(2);
            document.getElementById('min' + chanNum).value = document.getElementById('val' + chanNum).innerHTML;
            transmitSettings();
        };
        minButton.id = "mb" + chanNum;
        minButton.innerHTML = '=';
        rowMin.appendChild(minButton);
        newChannel.appendChild(rowMin);
        
        var rowValue = document.createElement("td");
        rowValue.innerHTML = '<span id="val'+chanNum+'"></span>';
        newChannel.appendChild(rowValue);
        
        var rowMax = document.createElement("td");
        var maxNumber = document.createElement("input");
        maxNumber.id = "max"+chanNum;
        maxNumber.type = "number";
        maxNumber.style.width = '64px';
        maxNumber.onchange = function() { transmitSettings(); };
        rowMax.appendChild(maxNumber);
        var maxButton = document.createElement("button");
        maxButton.onclick = function()
        {
            var chanNum = this.id.substring(2);
            document.getElementById('max' + chanNum).value = document.getElementById('val'+chanNum).innerHTML;
            transmitSettings();
        };
        maxButton.id = "xb" + chanNum;
        maxButton.innerHTML = '=';
        rowMax.appendChild(maxButton);
        newChannel.appendChild(rowMax);
        
        var rowInverted = document.createElement("td");
        var invCheck = document.createElement("input");
        invCheck.id = "inv"+chanNum;
        invCheck.type = "checkbox";
        invCheck.onchange = function() { transmitSettings(); };
        rowInverted.appendChild(invCheck);
        newChannel.appendChild(rowInverted);
        
        var rowOutput = document.createElement("td");
        rowOutput.innerHTML = '';
        rowOutput.id = "out"+chanNum;
        newChannel.appendChild(rowOutput);
        
        document.getElementById('channelTable').appendChild(newChannel);
    }
    connect();
};


